@@ -29,12 +29,9 @@
 #include <stdbool.h>
 #include <errno.h>
 #include "libevdev.h"
+#include "libevdev-util.h"
 
-#define LONG_BITS (sizeof(long) * 8)
-#define NLONGS(x) (((x) + LONG_BITS - 1) / LONG_BITS)
-#define ARRAY_LENGTH(a) (sizeof(a) / (sizeof((a)[0])))
 #define MAX_NAME 256
-#define MAX_SLOTS 32
 #define ABS_MT_MIN ABS_MT_SLOT
 #define ABS_MT_MAX ABS_MT_TOOL_Y
 #define ABS_MT_CNT (ABS_MT_MAX - ABS_MT_MIN + 1)
@@ -42,19 +39,6 @@
 #define LIBEVDEV_PRINTF(_format, _args) __attribute__ ((format (printf, _format, _args)))
 #define ALIAS(_to) __attribute__((alias(#_to)))
 
-#undef min
-#undef max
-#define min(a,b) \
-		({ __typeof__ (a) _a = (a); \
-	          __typeof__ (b) _b = (b); \
-		_a > _b ? _b : _a; \
-		})
-#define max(a,b) \
-		({ __typeof__ (a) _a = (a); \
-	          __typeof__ (b) _b = (b); \
-		_a > _b ? _a : _b; \
-		})
-
 /**
  * Sync state machine:
  * default state: SYNC_NONE
@@ -72,6 +56,11 @@ enum SyncState {
 	SYNC_IN_PROGRESS,
 };
 
+struct mt_sync_state {
+	int code;
+	int val[];
+};
+
 struct libevdev {
 	int fd;
 	bool initialized;
@@ -95,7 +84,7 @@ struct libevdev {
 	unsigned long led_values[NLONGS(LED_CNT)];
 	unsigned long sw_values[NLONGS(SW_CNT)];
 	struct input_absinfo abs_info[ABS_CNT];
-	int mt_slot_vals[MAX_SLOTS][ABS_MT_CNT];
+	int *mt_slot_vals; /* [num_slots * ABS_MT_CNT] */
 	int num_slots; /**< valid slots in mt_slot_vals */
 	int current_slot;
 	int rep_values[REP_CNT];
@@ -109,6 +98,15 @@ struct libevdev {
 	size_t queue_nsync; /**< number of sync events */
 
 	struct timeval last_event_time;
+
+	struct {
+		struct mt_sync_state *mt_state;
+		size_t mt_state_sz;		 /* in bytes */
+		unsigned long *slot_update;
+		size_t slot_update_sz;		 /* in bytes */
+		unsigned long *tracking_id_changes;
+		size_t tracking_id_changes_sz;	 /* in bytes */
+	} mt_sync;
 };
 
 struct logdata {
@@ -279,5 +277,60 @@ queue_set_num_elements(struct libevdev *dev, size_t nelem)
 
 	return 0;
 }
+
+#define max_mask(uc, lc) \
+	case EV_##uc: \
+			*mask = dev->lc##_bits; \
+			max = libevdev_event_type_get_max(type); \
+		break;
+
+
+static inline int
+type_to_mask_const(const struct libevdev *dev, unsigned int type, const unsigned long **mask)
+{
+	int max;
+
+	switch(type) {
+		max_mask(ABS, abs);
+		max_mask(REL, rel);
+		max_mask(KEY, key);
+		max_mask(LED, led);
+		max_mask(MSC, msc);
+		max_mask(SW, sw);
+		max_mask(FF, ff);
+		max_mask(REP, rep);
+		max_mask(SND, snd);
+		default:
+		     max = -1;
+		     break;
+	}
+
+	return max;
+}
+
+static inline int
+type_to_mask(struct libevdev *dev, unsigned int type, unsigned long **mask)
+{
+	int max;
+
+	switch(type) {
+		max_mask(ABS, abs);
+		max_mask(REL, rel);
+		max_mask(KEY, key);
+		max_mask(LED, led);
+		max_mask(MSC, msc);
+		max_mask(SW, sw);
+		max_mask(FF, ff);
+		max_mask(REP, rep);
+		max_mask(SND, snd);
+		default:
+		     max = -1;
+		     break;
+	}
+
+	return max;
+}
+
+#undef max_mask
 #endif
 
