@@ -73,7 +73,10 @@ START_TEST(test_init_and_change_fd)
 	dev = libevdev_new();
 	ck_assert(dev != NULL);
 	ck_assert_int_eq(libevdev_set_fd(dev, -1), -EBADF);
+
+	libevdev_set_log_function(test_logfunc_ignore_error, NULL);
 	ck_assert_int_eq(libevdev_change_fd(dev, -1), -1);
+	libevdev_set_log_function(test_logfunc_abort_on_error, NULL);
 
 	rc = uinput_device_new_with_events(&uidev,
 					   TEST_DEVICE_NAME, DEFAULT_IDS,
@@ -87,7 +90,10 @@ START_TEST(test_init_and_change_fd)
 					   -1);
 	ck_assert_msg(rc == 0, "Failed to create uinput device: %s", strerror(-rc));
 	ck_assert_int_eq(libevdev_set_fd(dev, uinput_device_get_fd(uidev)), 0);
+
+	libevdev_set_log_function(test_logfunc_ignore_error, NULL);
 	ck_assert_int_eq(libevdev_set_fd(dev, 0), -EBADF);
+	libevdev_set_log_function(test_logfunc_abort_on_error, NULL);
 
 	ck_assert_int_eq(libevdev_get_fd(dev), uinput_device_get_fd(uidev));
 
@@ -99,7 +105,6 @@ START_TEST(test_init_and_change_fd)
 }
 END_TEST
 
-
 static int log_fn_called = 0;
 static char *logdata = "test";
 static void logfunc(enum libevdev_log_priority priority,
@@ -113,6 +118,11 @@ static void logfunc(enum libevdev_log_priority priority,
 START_TEST(test_log_init)
 {
 	struct libevdev *dev = NULL;
+	enum libevdev_log_priority old;
+
+	old = libevdev_get_log_priority();
+
+	libevdev_set_log_priority(LIBEVDEV_LOG_DEBUG);
 
 	libevdev_set_log_function(logfunc, NULL);
 	libevdev_set_log_function(NULL, NULL);
@@ -136,6 +146,91 @@ START_TEST(test_log_init)
 	ck_assert_int_eq(log_fn_called, 2);
 
 	libevdev_free(dev);
+
+	libevdev_set_log_function(test_logfunc_abort_on_error, NULL);
+
+	log_fn_called = 0;
+
+	libevdev_set_log_priority(old);
+}
+END_TEST
+
+START_TEST(test_log_default_priority)
+{
+	ck_assert_int_eq(libevdev_get_log_priority(), LIBEVDEV_LOG_INFO);
+}
+END_TEST
+
+START_TEST(test_log_set_get_priority)
+{
+	enum libevdev_log_priority pri;
+	enum libevdev_log_priority old;
+
+	old = libevdev_get_log_priority();
+
+	pri = LIBEVDEV_LOG_DEBUG;
+	libevdev_set_log_priority(pri);
+	ck_assert_int_eq(libevdev_get_log_priority(), pri);
+
+	pri = LIBEVDEV_LOG_INFO;
+	libevdev_set_log_priority(pri);
+	ck_assert_int_eq(libevdev_get_log_priority(), pri);
+
+	pri = LIBEVDEV_LOG_ERROR;
+	libevdev_set_log_priority(pri);
+	ck_assert_int_eq(libevdev_get_log_priority(), pri);
+
+	/* debug and above is clamped */
+	pri = LIBEVDEV_LOG_DEBUG + 1;
+	libevdev_set_log_priority(pri);
+	ck_assert_int_eq(libevdev_get_log_priority(), LIBEVDEV_LOG_DEBUG);
+
+	/*  error and below is not clamped, we need this for another test */
+	pri = LIBEVDEV_LOG_ERROR - 1;
+	libevdev_set_log_priority(pri);
+	ck_assert_int_eq(libevdev_get_log_priority(), pri);
+
+	libevdev_set_log_priority(old);
+}
+END_TEST
+
+START_TEST(test_log_priority)
+{
+	struct libevdev *dev = NULL;
+	enum libevdev_log_priority old;
+
+	old = libevdev_get_log_priority();
+
+	libevdev_set_log_function(logfunc, logdata);
+
+	dev = libevdev_new();
+	ck_assert(dev != NULL);
+
+	libevdev_set_log_priority(LIBEVDEV_LOG_DEBUG);
+	libevdev_next_event(dev, LIBEVDEV_READ_FLAG_NORMAL, NULL);
+	ck_assert_int_eq(log_fn_called, 1);
+
+	libevdev_set_log_priority(LIBEVDEV_LOG_INFO);
+	libevdev_next_event(dev, LIBEVDEV_READ_FLAG_NORMAL, NULL);
+	ck_assert_int_eq(log_fn_called, 2);
+
+	libevdev_set_log_priority(LIBEVDEV_LOG_ERROR);
+	libevdev_next_event(dev, LIBEVDEV_READ_FLAG_NORMAL, NULL);
+	ck_assert_int_eq(log_fn_called, 3);
+
+	/* we don't have any log msgs > ERROR at the moment, so test it by
+	   setting an invalid priority. */
+	libevdev_set_log_priority(LIBEVDEV_LOG_ERROR - 1);
+	libevdev_next_event(dev, LIBEVDEV_READ_FLAG_NORMAL, NULL);
+	ck_assert_int_eq(log_fn_called, 3);
+
+	libevdev_free(dev);
+
+	libevdev_set_log_function(test_logfunc_abort_on_error, NULL);
+
+	log_fn_called = 0;
+
+	libevdev_set_log_priority(old);
 }
 END_TEST
 
@@ -236,21 +331,22 @@ START_TEST(test_device_grab)
 	struct libevdev *dev;
 	int rc;
 
-	rc = test_create_device(&uidev, &dev,
-				EV_SYN, SYN_REPORT,
-				EV_REL, REL_X,
-				EV_REL, REL_Y,
-				EV_REL, REL_WHEEL,
-				EV_KEY, BTN_LEFT,
-				EV_KEY, BTN_MIDDLE,
-				EV_KEY, BTN_RIGHT,
-				-1);
-	ck_assert_msg(rc == 0, "Failed to create device: %s", strerror(-rc));
+	test_create_device(&uidev, &dev,
+			   EV_SYN, SYN_REPORT,
+			   EV_REL, REL_X,
+			   EV_REL, REL_Y,
+			   EV_REL, REL_WHEEL,
+			   EV_KEY, BTN_LEFT,
+			   EV_KEY, BTN_MIDDLE,
+			   EV_KEY, BTN_RIGHT,
+			   -1);
 
+	libevdev_set_log_function(test_logfunc_ignore_error, NULL);
 	rc = libevdev_grab(dev, 0);
 	ck_assert_int_eq(rc, -EINVAL);
 	rc = libevdev_grab(dev, 1);
 	ck_assert_int_eq(rc, -EINVAL);
+	libevdev_set_log_function(test_logfunc_abort_on_error, NULL);
 
 	rc = libevdev_grab(dev, LIBEVDEV_UNGRAB);
 	ck_assert_int_eq(rc, 0);
@@ -272,16 +368,15 @@ START_TEST(test_set_clock_id)
 	struct libevdev *dev;
 	int rc;
 
-	rc = test_create_device(&uidev, &dev,
-				EV_SYN, SYN_REPORT,
-				EV_REL, REL_X,
-				EV_REL, REL_Y,
-				EV_REL, REL_WHEEL,
-				EV_KEY, BTN_LEFT,
-				EV_KEY, BTN_MIDDLE,
-				EV_KEY, BTN_RIGHT,
-				-1);
-	ck_assert_msg(rc == 0, "Failed to create device: %s", strerror(-rc));
+	test_create_device(&uidev, &dev,
+			   EV_SYN, SYN_REPORT,
+			   EV_REL, REL_X,
+			   EV_REL, REL_Y,
+			   EV_REL, REL_WHEEL,
+			   EV_KEY, BTN_LEFT,
+			   EV_KEY, BTN_MIDDLE,
+			   EV_KEY, BTN_RIGHT,
+			   -1);
 
 	rc = libevdev_set_clock_id(dev, CLOCK_REALTIME);
 	ck_assert_int_eq(rc, 0);
@@ -307,16 +402,15 @@ START_TEST(test_clock_id_events)
 	struct timespec t1_mono, t2_mono;
 	int64_t t1, t2;
 
-	rc = test_create_device(&uidev, &dev,
-				EV_SYN, SYN_REPORT,
-				EV_REL, REL_X,
-				EV_REL, REL_Y,
-				EV_REL, REL_WHEEL,
-				EV_KEY, BTN_LEFT,
-				EV_KEY, BTN_MIDDLE,
-				EV_KEY, BTN_RIGHT,
-				-1);
-	ck_assert_msg(rc == 0, "Failed to create device: %s", strerror(-rc));
+	test_create_device(&uidev, &dev,
+			   EV_SYN, SYN_REPORT,
+			   EV_REL, REL_X,
+			   EV_REL, REL_Y,
+			   EV_REL, REL_WHEEL,
+			   EV_KEY, BTN_LEFT,
+			   EV_KEY, BTN_MIDDLE,
+			   EV_KEY, BTN_RIGHT,
+			   -1);
 
 	fd = open(uinput_device_get_devnode(uidev), O_RDONLY);
 	ck_assert_int_gt(fd, -1);
@@ -365,7 +459,6 @@ START_TEST(test_clock_id_events)
 }
 END_TEST
 
-
 Suite *
 libevdev_init_test(void)
 {
@@ -380,6 +473,9 @@ libevdev_init_test(void)
 
 	tc = tcase_create("log init");
 	tcase_add_test(tc, test_log_init);
+	tcase_add_test(tc, test_log_priority);
+	tcase_add_test(tc, test_log_set_get_priority);
+	tcase_add_test(tc, test_log_default_priority);
 	tcase_add_test(tc, test_log_data);
 	suite_add_tcase(s, tc);
 
